﻿using System;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Windows.Forms.Design.Behavior;
using Infragistics.Shared;
using Infragistics.Win;
using Infragistics.Win.UltraWinMaskedEdit;
using Infragistics.Win.UltraWinEditors;

namespace NumericUserControlProperties.Shared.BaseClasses
{
    [System.Security.Permissions.PermissionSet(System.Security.Permissions.SecurityAction.Demand, Name = "FullTrust")]
    public class NumericEditorBaseDesigner : ControlDesigner
    {
        #region .   Variables   .
        // References to designer services, for convenience.
        private IComponentChangeService changeService = null;
        private ISelectionService selectionService = null;
        private BehaviorService behaviorSvc = null;

        // This is the collection of DesignerActionLists that
        // defines the smart tags offered on the control. 
        private DesignerActionListCollection actionLists = null;
        #endregion

        #region .   Numeric Editor Base Designer   .
        public NumericEditorBaseDesigner()
        {   
        }

        #region . Dispose   .
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
            }

            base.Dispose(disposing);
        }
        #endregion

        #region . Initialize   .
        // This method is where the designer initializes its state when
        // it is created.
        public override void Initialize(IComponent component)
        {
            base.Initialize(component);

            // Connect to various designer services.
            InitializeServices();
        }

        #region . New Component   .
        // This demonstrates changing the appearance of a control while
        // it is being designed. In this case, the BackColor property is
        // set to LightBlue. 
        public override void InitializeNewComponent(IDictionary defaultValues)
        {
            base.InitializeNewComponent(defaultValues);
        }
        #endregion

        #region . Services   .
        // This utility method connects the designer to various services.
        // These references are cached for convenience.
        private void InitializeServices()
        {
            // Acquire a reference to IComponentChangeService.
            this.changeService =
                GetService(typeof(IComponentChangeService))
                as IComponentChangeService;

            // Acquire a reference to ISelectionService.
            this.selectionService =
                GetService(typeof(ISelectionService))
                as ISelectionService;

            // Acquire a reference to BehaviorService.
            this.behaviorSvc =
                GetService(typeof(BehaviorService))
                as BehaviorService;
        }
        #endregion

        #endregion

        #endregion

        #region .   Designer Action List   .
        // This method creates the DesignerActionList on demand, causing
        // smart tags to appear on the control being designed.
        public override DesignerActionListCollection ActionLists
        {
            get
            {
                if (null == actionLists)
                {
                    actionLists = new DesignerActionListCollection();

                    actionLists.Add(
                        new ActionList(this.Component));
                }

                return actionLists;
            }
        }
        #endregion

        public class ActionList : System.ComponentModel.Design.DesignerActionList
        {
            #region .   Action List   .
            // Cache a reference to the control.
            private NumericEditorBase relatedControl;

            //The constructor associates the control 
            //with the smart tag list.
            public ActionList(IComponent component)
                : base(component)
            {
                this.relatedControl = component as NumericEditorBase;
            }
            #endregion

            #region .   Variables   .

            #region .   Use Os Themes   .
            // Properties that are targets of DesignerActionPropertyItem entries.
            public DefaultableBoolean UseOsThemes
            {
                get
                {
                    return this.relatedControl.UseOsThemes;
                }
                set
                {
                    PropertyDescriptor pdAnchor = TypeDescriptor.GetProperties(this.relatedControl)["UseOsThemes"];
                    pdAnchor.SetValue(this.relatedControl, value);
                }
            }
            #endregion

            #region .   Mask Input   .
            // Properties that are targets of DesignerActionPropertyItem entries.
            [Editor(typeof(MaskUITypeEditor), typeof(UITypeEditor))]
            [TypeConverter(typeof(MaskTypeConverter))]
            public string MaskInput
            {
                get
                {
                    return this.relatedControl.MaskInput;
                }
                set
                {
                    PropertyDescriptor pdAnchor = TypeDescriptor.GetProperties(this.relatedControl)["MaskInput"];
                    pdAnchor.SetValue(this.relatedControl, value);
                }
            }
            #endregion

            #region .   Numeric Type   .
            // Properties that are targets of DesignerActionPropertyItem entries.
            public NumericType NumericType
            {
                get
                {
                    return this.relatedControl.NumericType;
                }
                set
                {
                    PropertyDescriptor pdAnchor = TypeDescriptor.GetProperties(this.relatedControl)["NumericType"];
                    pdAnchor.SetValue(this.relatedControl, value);
                }
            }
            #endregion

            #region .   Minimum Value   .
            // Properties that are targets of DesignerActionPropertyItem entries.
            [TypeConverter(typeof(StringConverter))]
            public object MinValue
            {
                get
                {
                    return this.relatedControl.MinValue;
                }
                set
                {
                    PropertyDescriptor pdAnchor = TypeDescriptor.GetProperties(this.relatedControl)["MinValue"];
                    pdAnchor.SetValue(this.relatedControl, value);
                }
            }
            #endregion

            #region .   Maximum Value   .
            // Properties that are targets of DesignerActionPropertyItem entries.
            [TypeConverter(typeof(StringConverter))]
            public object MaxValue
            {
                get
                {
                    return this.relatedControl.MaxValue;
                }
                set
                {
                    PropertyDescriptor pdAnchor = TypeDescriptor.GetProperties(this.relatedControl)["MaxValue"];
                    pdAnchor.SetValue(this.relatedControl, value);
                }
            }
            #endregion

            #endregion

            #region .   Get Sorted Action Items   .
            // This method creates and populates the DesignerActionItemCollection which is used to 
            // display smart tag items.
            public override DesignerActionItemCollection GetSortedActionItems()
            {
                DesignerActionItemCollection items = new DesignerActionItemCollection();

                #region .   Appearance   .

                #region .   Use OS Themes   .
                items.Add(new DesignerActionPropertyItem(
                    "UseOsThemes",
                    "UseOsThemes:",
                    "Appearance",
                    "Indicates whether the elements of a control may be rendered using the operating system theme rendering."));
                #endregion

                #endregion

                #region .   Behavior   .

                #region .   Mask Input   .
                items.Add(new DesignerActionPropertyItem(
                    "MaskInput",
                    "Mask Input:",
                    "Behavior",
                    "Gets/sets the input mask used by the control."));
                #endregion

                #region .   Numeric Type   .
                items.Add(new DesignerActionPropertyItem(
                    "NumericType",
                    "Numeric Type:",
                    "Behavior",
                    "Gets/sets the type of numeric data the control supports."));
                #endregion

                #region .   Min Value   .
                items.Add(new DesignerActionPropertyItem(
                    "MinValue",
                    "Minimum Value:",
                    "Behavior",
                    "Gets/sets the control's minimum allowable value."));
                #endregion

                #region .   Max Value   .
                items.Add(new DesignerActionPropertyItem(
                    "MaxValue",
                    "Maximum Value:",
                    "Behavior",
                    "Gets/sets the control's maximum allowable value."));
                #endregion

                #endregion

               return items;
            }
            #endregion
        }
    }
}